<?php

/**
 * API for vps.net 
 *
 * This API provides an interface to vps.net allowing common virtual machine and account management tasks 
 * @package VPSNET
 * @version 1.0.4
 *
 *
 * Changelog:
 * 2009-06-10 Added proxy support, fixed incorrect parameters (hostname+domain_name) passed in create virtual machine - now uses fqdn
 * 2009-06-02 Fixed showConsole function 
 * 2009-06-02 Fixed graph function
 * 2009-05-31 Fixed CURL_USERAGENT to CURLOPT_USERAGENT
 * 2009-05-29 Added changelog, fixed API resource for available clouds.
 */
class VPSNET {

    /**
     * This contains the API URL for accessing vps.net. You should not
     * need to change this unless asked to do so by vps.net support.
     * @var string
     *
     * */
    protected $_apiUrl = 'https://control.vps.net';
    /**
     * This contains the API version and is sent as part of server
     * requests.
     * @var string
     */
    private $_apiVersion = 'api10json';
    protected $_apiUserAgent = "VPSNET_API_10_JSON/PHP";
    private $_session_cookie;
    private $_auth_name = '';
    private $_auth_api_key = '';
    private $ch = null;
    private $_proxy = '';
    private static $instance;

    private function __construct() {

    }

    public function __destruct() {
        if (!is_null($this->ch)) {
            curl_close($this->ch);
        }
    }

    /**
     * Returns true if the API instance has authentication information set.
     * If not, you can call getInstance() with credentials.
     * @return boolean
     */
    public function isAuthenticationInfoSet() {
        return (strlen($this->_auth_name) > 0 && strlen($this->_auth_api_key) > 0);
    }

    /**
     * Returns the instance of the API.
     * @return VPSNET
     */
    public static function getInstance($username = '', $_auth_api_key = '', $proxy = '') {
        if (!isset(self::$instance)) {
            $c = __CLASS__;
            self::$instance = new $c();
            self::$instance->_auth_name = $username;
            self::$instance->_auth_api_key = $_auth_api_key;
            if (strlen($proxy) > 0) {
                self::$instance->_proxy = $proxy;
            }
            if ((strlen($username) == 0 || strlen($_auth_api_key) == 0)) {
                trigger_error('Singleton has been called for the first time and a username or API Key has not been set.', E_USER_ERROR);
            }
            self::$instance->_initCurl();
        }
        return self::$instance;
    }

    public function __clone() {
        trigger_error('Clone is not permitted. This class is a singleton.', E_USER_ERROR);
    }

    private function _initCurl() {
        $this->ch = curl_init();
        if (strlen($this->_proxy) > 0) {
            curl_setopt($this->ch, CURLOPT_PROXY, $this->_proxy);
        }
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Accept: application/json'));
        curl_setopt($this->ch, CURLOPT_USERAGENT, $this->_apiUserAgent);
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYHOST, 1);
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->ch, CURLOPT_USERPWD, $this->_auth_name . ':' . $this->_auth_api_key);
        curl_setopt($this->ch, CURLOPT_COOKIEFILE, "/tmp/.vpsnet.cookie");
        curl_setopt($this->ch, CURLOPT_COOKIEJAR, "/tmp/.vpsnet.cookie");
    }

    public function setAPIResource($resource, $append_api_version = true, $queryString = '') {
        if ($append_api_version) {
//			curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s.%3$s?%4$s', $this->_apiUrl, $resource, $this->_apiVersion, $queryString));
            if ($queryString)
                curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s.%3$s?%4$s', $this->_apiUrl, $resource, $this->_apiVersion, $queryString));
            else
                curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s.%3$s', $this->_apiUrl, $resource, $this->_apiVersion));
        } else {
//			curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s?%3$s', $this->_apiUrl, $resource, $queryString));
            if ($queryString)
                curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s?%3$s', $this->_apiUrl, $resource, $queryString));
            else
                curl_setopt($this->ch, CURLOPT_URL, sprintf('%1$s/%2$s', $this->_apiUrl, $resource));
        }
    }

    public function sendGETRequest() {
        curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($this->ch, CURLOPT_HTTPGET, true);
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Accept: application/json'));
        return $this->sendRequest();
    }

    public function sendPOSTRequest($data = null, $encodeasjson = true) {
        curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'POST');

        if ($encodeasjson) {
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, json_encode($data));
        } else {
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $data);
        }
        curl_setopt($this->ch, CURLOPT_POST, true);
		curl_setopt($this->ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Accept: application/json'));

        $rtn = $this->sendRequest();

        return $rtn;
    }

    public function sendPUTRequest($data) {
        curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'PUT');
        $json_data = json_encode($data);
        curl_setopt($this->ch, CURLOPT_POSTFIELDS, $json_data);
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, array('Content-Length: ' . strlen($json_data), 'Content-Type: application/json', 'Accept: application/json'));
        return $this->sendRequest();
    }

    public function sendDELETERequest() {
        curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Accept: application/json'));
        return $this->sendRequest();
    }

    protected function sendRequest($data = null) {
        $rtn = array();

        $rtn['response_body'] = curl_exec($this->ch);
        $rtn['info'] = curl_getinfo($this->ch);

        if (isset($rtn['info']['content_type']) && $rtn['info']['content_type'] == 'application/json; charset=utf-8') {
            if ($rtn['info']['http_code'] == 200) {
                $rtn['response'] = json_decode($rtn['response_body']);
            } else if ($rtn['info']['http_code'] == 422) {
                $rtn['errors'] = json_decode($rtn['response_body']);
            }
        }
        return $rtn;
    }

    public function getCloud($id) {

        $this->setAPIResource('clouds/' . $id);
        $result = $this->sendGETRequest();

        return $result['response'];
    }

    public function getClauds($all=false) {

        $this->setAPIResource('available_clouds');
        $result = $this->sendGETRequest();

        $return = array();

        foreach ($result['response'] as $cloud) {

            //print_r($cloud);

            if ($all)
                $return[] = array('id' => $cloud->cloud->id, 'version' => $cloud->cloud->cloud_version, 'label' => $cloud->cloud->label, 'available' => $cloud->cloud->available);
            elseif ($cloud->cloud->available) {
                $return[] = array('id' => $cloud->cloud->id, 'version' => $cloud->cloud->cloud_version, 'label' => $cloud->cloud->label, 'available' => $cloud->cloud->available);
            }
        }

        return $return;
    }

    public function getTemplateName($id) {

        if ($this->AllTemplates[$id])
            return $this->AllTemplates[$id];


        $this->setAPIResource('available_templates');

        $result = $this->sendGETRequest();

        foreach ($result['response']->template_groups as $template_group) {
            foreach ($template_group->templates as $tempalte) {
                $this->AllTemplates[$tempalte->clouds[0]->system_template_id] = $tempalte->name;
            }
        }

        return $this->AllTemplates[$id];
        //print_r($this->AllTemplates[]);
    }

    public function getAllTemplatesGroups() {

        $this->setAPIResource('available_templates');

        $result = $this->sendGETRequest();

        $this->AllTemplatesInfo = array();


        $return = array();

        foreach ($result['response']->template_groups as $template_group) {
            $return[] = $template_group->name;
            //print_r($template_group);
        }


        return $return;
    }

    public function getAllTemplates($group=false) {

        $this->setAPIResource('available_templates');

        $result = $this->sendGETRequest();

        $this->AllTemplatesInfo = array();

        foreach ($result['response']->template_groups as $template_group) {

            foreach ($template_group->templates as $tempalte) {
                //print_r($tempalte);

                if ($group) {

                    if ($group == $template_group->name) {
                        if (!isset($this->AllTemplatesInfo[$tempalte->clouds[0]->system_template_id])) {

                            $this->AllTemplatesInfo[$tempalte->clouds[0]->system_template_id] = $this->getTemplateInfo($tempalte->clouds[0]->id, $tempalte->clouds[0]->system_template_id);
                        }
                    }
                } else {

                    if (!isset($this->AllTemplatesInfo[$tempalte->clouds[0]->system_template_id])) {

                        $this->AllTemplatesInfo[$tempalte->clouds[0]->system_template_id] = $this->getTemplateInfo($tempalte->clouds[0]->id, $tempalte->clouds[0]->system_template_id);
                    }
                }
            }
        }

        return $this->AllTemplatesInfo;
    }

    /**
     * Assign customer id to unassigned nodes
     */
    /*        public function pickupNodes ($consumer_id,$count,$type='vps') {

      $all=$this->getNodes(0,$type);

      print_r($all);

      }
     */

    /**
     * Returns Nodes from your account.
     * @param int $consumer_id (Optional) Consumer Id to filter results by
     * @return array An array of IPAddress instances
     */
    public function getNodes($consumer_id = 0, $type='vps') {
        if ($consumer_id > 0) {
            $this->setAPIResource('nodes', true, 'consumer_id=' . $consumer_id . '&type=' . $type);
        } else {
            $this->setAPIResource('nodes', true, 'type=' . $type);
        }

        $result = $this->sendGETRequest();

        $return = array();
        if ($result['info']['http_code'] == 422) {
            
        } else if ($result['response']) {
            $response = $result['response'];
            for ($x = 0; $x < count($response); $x++) {

                if ($response[$x]->slice->id) {
                    $return[$x] = new Node($response[$x]->slice->id, $response[$x]->slice->virtual_machine_id, $response[$x]->slice->consumer_id, 'vps', (bool) $response[$x]->slice->free_node, (bool) $response[$x]->slice->discount_node);
                } elseif ($response[$x]->storage_node->id) {
                    $return[$x] = new Node($response[$x]->storage_node->id, $response[$x]->storage_node->virtual_machine_id, $response[$x]->storage_node->consumer_id, 'storage', (bool) $response[$x]->storage_node->free_node, (bool) $response[$x]->storage_node->discount_node);
                } elseif ($response[$x]->ram_node->id) {
                    $return[$x] = new Node($response[$x]->ram_node->id, $response[$x]->ram_node->virtual_machine_id, $response[$x]->ram_node->consumer_id, 'ram', (bool) $response[$x]->ram_node->free_node, (bool) $response[$x]->ram_node->discount_node);
                }
            }
        }
        return $return;
    }

    /**
     * Returns IP addresses from your account.
     * @param int $consumer_id (Optional) Consumer Id to filter results by
     * @return array An array of IPAddress instances
     */
    public function getIPAddresses($consumer_id = 0) {
        if ($consumer_id > 0) {
            $this->setAPIResource('ips', true, 'consumer_id=' . $consumer_id);
            //$this->setAPIResource('ip_address_assignments', true, 'consumer_id=' . $consumer_id);
        } else {
            $this->setAPIResource('ips');
            //$this->setAPIResource('ip_address_assignments');
        }
        $result = $this->sendGETRequest();
        $return = array();
        if ($result['info']['http_code'] == 422) {
            
        } else if ($result['response']) {
            $response = $result['response'];
//                        print_r($result);
            for ($x = 0; $x < count($response); $x++) {
                $return[$x] = $this->_castObjectToClass('IPAddress', $response[$x]->ip_address);
            }
        }
        return $return;
    }

    /**
     * Returns Virtual Machines from your account.
     * @param int $consumer_id (Optional) Consumer Id to filter results by
     * @return array An array of VirtualMachine instances
     */
    public function getVirtualMachines($consumer_id = 0, $full = true) {

        $names = array();
        if($full){
            $clouds = $this->getAvailableCloudsAndTemplates();
            foreach ($clouds as $info) {
                $names[$info->cloud->id] = $info->cloud->label;
            }
        }
        //print_r($names);
        $furl = $full ? '' : 'basic=true';
        if ($consumer_id > 0)
            $this->setAPIResource('virtual_machines', true, 'consumer_id=' . $consumer_id.''.(!empty($furl)?'&':''). $furl);
        else
            $this->setAPIResource('virtual_machines', true, $furl);

        $result = $this->sendGETRequest();
        
        $return = array();
        if ($result['info']['http_code'] == 422) {
            
        } else if ($result['response']) {
            $response = $result['response'];

//                        print_r($response);
            for ($x = 0; $x < count($response); $x++) {
                
                if($clouds)
                    $response[$x]->virtual_machine->cloud_label = $names[$response[$x]->virtual_machine->cloud_id];

                $return[$x] = $this->_castObjectToClass('VirtualMachine', $response[$x]->virtual_machine);
            }
        }
        return $return;
    }

    /**
     * Returns available Clouds and Virtual Machine templates.
     * @return array
     */
    public function getAvailableCloudsAndTemplates() {
        $this->setAPIResource('available_clouds');
        $result = $this->sendGETRequest();
        $return = null;
        if ($result['info']['http_code'] == 422) {
            
        } else if ($result['response']) {
            $return = $result['response'];
        }
        return $return;
    }

    public function getCloudsTemplatesGroups($cloud_id) {

        $this->setAPIResource('available_templates');
        $result = $this->sendGETRequest();
        $return = array();

        if ($result['info']['http_code'] != 200) {
            return;
        } else if ($result['response']) {
//			$return = $result['response'];
        }


        foreach ($result['response']->template_groups as $group) {

            $available = false;

//                       print_r($group);

            foreach ($group->templates as $template) {

                foreach ($template->clouds as $claud) {
//                                    print_r($claud);
//                                  echo $claud->id."\n";
                    if ($claud->id == $cloud_id)
                        $available = true;;
                }
            };


            if ($available)
                $return[] = $group->name;
        }

        sort($return);
//                print_r($return);
        return $return;
    }

    public function isWindowsTemplate($sid) {
        $this->setAPIResource('windows_templates');
        $result = $this->sendGETRequest();
        foreach ($result['response'] as $templ) {
            if ($templ->windows_template->virtual_machine_id == $sid) {
                return true;
            }
        }
        return false;
    }

    public function getTemplateInfo($cloud_id, $tid) {
        //clouds/20/system_templates/942.api10json
        $this->setAPIResource('clouds/' . $cloud_id . '/system_templates/' . $tid);
        $result = $this->sendGETRequest();

        return $result['response']->system_template;
    }

    public function getCloudsTemplates($cloud_id, $groupname) {

        $this->setAPIResource('available_templates');
        $result = $this->sendGETRequest();
        $return = array();

        if ($result['info']['http_code'] != 200) {
            return;
        };

        $return = array();

//                print_r($result['response']);

        foreach ($result['response']->template_groups as $group) {

            if ($group->name == $groupname) {
//                        print_r($group);

                foreach ($group->templates as $template) {
//                            print_r($template);

                    foreach ($template->clouds as $claud) {

//                                  echo $claud->id."\n";
                        if ($claud->id == $cloud_id) {
//                                      echo $template->name."\n";
//                                      print_r($claud);
                            $return[$claud->system_template_id] = $template->name;
                        }
                    }
                }
            };
//                        print_r($group->templates);
        }
        //echo $group;
//        print_r($return);
        return $return;
    }

    public function deleteIP($id) {

        $this->setAPIResource('ips/' . $id);

        $result = $this->sendDELETERequest();

        return $result['response'];
    }

    /**
     * Adds IP addresses to your account.
     * @return IPAddress An instance of the IP address that was assigned
     */
    public function addIPAddress($virtual_machine_id, $type) {

        $this->setAPIResource('ips');

        $requestdata = array();
        $requestdata['type'] = $type;
        $requestdata['quantity'] = 1;
        $requestdata['virtual_machine_id'] = $virtual_machine_id;
//                $requestdata['consumer_id']        = $virtual_machine_id;
//                $requestdata['description']        = 'for virtual machine #'.$virtual_machine_id;

        $result = $this->sendPOSTRequest($requestdata);

//                print_r($result);

        $ip = $result['response'][0]->ip_address;

        $requestdata = array();

        if ($ip) {

            $this->setAPIResource('ips/' . $result['response'][0]->ip_address->id);

            $requestdata['ip_address'] = array('notes' => 'for virtual machine #' . $virtual_machine_id);

            $result = $this->sendPUTRequest($requestdata);

            //echo "edit....";
            //print_r($result);
        }


        return $ip;
    }

    /**
     * Creates a new Virtual Machine account.
     * @param VirtualMachine $virtualmachine Instance of VirtualMachine containing new virtual machine properties
     * @return VirtualMachine|object An instance of the created VirtualMachine that was assigned or an Object of errors
     */
    public function createVirtualMachine($virtualmachine) {
        $this->setAPIResource('virtual_machines');
        $requestdata['label'] = $virtualmachine->label;
        $requestdata['fqdn'] = $virtualmachine->hostname;
        // $requestdata['domain_name'] = $virtualmachine->domain_name;
        $requestdata['slices_required'] = $virtualmachine->slices_required;
        $requestdata['storage_nodes_required'] = $virtualmachine->storage_nodes_required;
        $requestdata['ram_nodes_required'] = $virtualmachine->ram_nodes_required;

        $requestdata['backups_enabled'] = (int) $virtualmachine->backups_enabled;
        $requestdata['rsync_backups_enabled'] = (int) $virtualmachine->rsync_backups_enabled;
        $requestdata['r1_soft_backups_enabled'] = (int) $virtualmachine->r1_soft_backups_enabled;

        $requestdata['system_template_id'] = $virtualmachine->system_template_id;
        $requestdata['cloud_id'] = $virtualmachine->cloud_id;
        $requestdata['consumer_id'] = $virtualmachine->consumer_id;
        $requestdata['auto_scaling'] = $virtualmachine->auto_scaling;
		
        if (is_array($virtualmachine->licence))
            $requestdata['licenses'] = $virtualmachine->licence;
        
        $json_request['virtual_machine'] = $requestdata;
        $result = $this->sendPOSTRequest($json_request);
        $return = null;

        if (isset($result['response'])) {
            $return = $this->_castObjectToClass('VirtualMachine', $result['response']);
            return $result['response'];
        } else {
            if (isset($result['errors']) && isset($result['errors']->errors) && count($result['errors']->errors) > 0) {
                $errors = array();
                foreach ($result['errors']->errors as $error) {
                    $errors[] = "{$error[0]} {$error[1]}";
                }
                $errors = implode(", ", $errors);
                throw new Exception($errors);
            }
            throw new Exception("Unknown error");
        }
    }

    /**
     * Adds Nodes to your account.
     * @param int $quantity	Number of Nodes to add
     * @param int $consumer_id (Optional) Consumer Id to tag the IP Address with
     * @return boolean true if nodes were added succesfully, false otherwise
     */
    public function addNodes($quantity, $consumer_id = 0, $type='vps') {
        if($consumer_id != 0){
            $api = VPSNET::getInstance();
            $resellersNodes = $api->getNodes(0, $type);
            foreach ($resellersNodes as $node) {
                if($node->virtual_machine_id == 0 && $node->consumer_id == 0){
                    $freeNodes[] = $node;
                }
            }
            if(count($freeNodes) > 0){
                for($n = 0; $n < min($quantity, count($freeNodes)); $n++){
                    $freeNodes[$n]->update($consumer_id);
                }
                $quantity -= $n;
            }
        }
        if($quantity > 0){
            $this->setAPIResource('nodes');
            $json_request['quantity'] = $quantity;

            if ($type == 'storage' || $type == 'ram') {

                $json_request['upgrade_option'] = $type;
            }

            if ($consumer_id > 0) {
                $json_request['consumer_id'] = $consumer_id;
            }
            $result = $this->sendPOSTRequest($json_request);
            return ($result['info']['http_code'] == 200);
        }

        return true;
    }

    public function _castObjectToClass($classname, $object) {
        return unserialize(preg_replace('/^O:\d+:"[^"]++"/', 'O:' . strlen($classname) . ':"' . $classname . '"', serialize($object)));
    }

}

/**
 * Node class 
 *
 * Allows management of Nodes
 */
class Node {

    public $virtual_machine_id = 0;
    public $id = 0;
    public $consumer_id = 0;
    public $deleted = 0;
    public $free = false;
    public $discount = false;
    public $type = 'vps';

    public function __construct($id = 0, $virtual_machine_id = 0, $consumer_id=0, $type='vps', $free = false, $discount = false) {
        $this->id = $id;
        $this->virtual_machine_id = $virtual_machine_id;
        $this->consumer_id = $consumer_id;
        $this->type = $type;
        $this->free = $free;
        $this->discount = $discount;
    }

    public function update($consumer_id) {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            trigger_error("To call Node::remove() you must set its id", E_USER_ERROR);
            return false;
        }

        switch ($this->type) {

            case 'storage':
                $api->setAPIResource('nodes/' . $this->id . '/update_storage_node');
                break;

            case 'ram':
                $api->setAPIResource('nodes/' . $this->id . '/update_ram_node');
                break;

            default:
                $api->setAPIResource('nodes/' . $this->id);
                break;
        }

        $json_request['consumer_id'] = $consumer_id;

        $result = $api->sendPUTRequest($json_request);

        return $result['info']['http_code'] ? true : false;
    }

    /**
     * Removes Node from your account
     * @return boolean true if Node was deleted succesfully, false otherwise
     */
    public function remove() {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            trigger_error("To call Node::remove() you must set its id", E_USER_ERROR);
            return false;
        }
        if ($this->virtual_machine_id > 0) {
            trigger_error("You cannot call Node::remove() with a node assigned to a virtual machine. Instead use VirtualMachine::update()", E_USER_ERROR);
            return false;
        }
        switch ($this->type) {

            case 'storage':
                $api->setAPIResource('nodes/' . $this->id . '/remove_storage_node');
                break;

            case 'ram':
                $api->setAPIResource('nodes/' . $this->id . '/remove_ram_node');
                break;

            default:
                $api->setAPIResource('nodes/' . $this->id);
                break;
        }

        $result = $api->sendDELETERequest();

        $this->deleted = ($result['info']['http_code'] == 200);
        return $this->deleted;
    }

}

/**
 * IP Address class 
 *
 * Allows management of IP addresses
 */
class IPAddress {

    public $id = 0;
    public $netmask = '';
    public $network = '';
    public $cloud_id = 0;
    public $ip_address = '';
    public $consumer_id = 0;
    public $deleted = false;

    private function __construct($id) {
        $this->id = $id;
    }

    /**
     * Use to find out if an IP address is Internal
     * @return boolean true if IP address is Internal, false otherwise
     */
    public function isInternal() {
        return ($cloud_id == 0);
    }

    /**
     * Use to find out if an IP address is External
     * @return boolean true if IP address is External, false otherwise
     */
    public function isExternal() {
        return ($cloud_id > 0);
    }

    /**
     * Removes IP address from your account
     * @return boolean true if IP address was deleted succesfully, false otherwise
     */
    public function remove() {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            trigger_error("To call IPAddress::remove() you must set id", E_USER_ERROR);
            return false;
        }
        $api->setAPIResource('ip_address_assignments/' . $this->id);
        $result = $api->sendDELETERequest();
        $this->deleted = ($result['info']['http_code'] == 200);
        return $this->deleted;
    }

}

/**
 * Backups class
 *
 * Allows management of Backups
 */
class Backup {

    public $virtual_machine_id = 0;
    public $id = 0;
    public $label = '';
    public $auto_backup_type;
    public $deleted = false;

    public function __construct($id = 0, $virtual_machine_id = 0) {
        $this->id = $id;
        $this->virtual_machine_id = $virtual_machine_id;
    }

    /**
     * Restores a backup
     * @return boolean true if backup restore request was succesful, false otherwise
     */
    public function restore() {
        $api = VPSNET::getInstance();
        if ($this->id < 1 || $this->virtual_machine_id < 1) {
            trigger_error("To call Backup::restore() you must set id and virtual_machine_id", E_USER_ERROR);
            return false;
        }
        $api->setAPIResource('virtual_machines/' . $this->virtual_machine_id . '/backups/' . $this->id . '/restore');
        $result = $api->sendPOSTRequest();
        if ($result['info']['http_code'] == 200) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Removes a backup
     * @return boolean true if backup was removed, false otherwise
     */
    public function remove() {
        $api = VPSNET::getInstance();
        if ($this->id < 1 || $this->virtual_machine_id < 1) {
            trigger_error("To call Backup::remove() you must set id and virtual_machine_id", E_USER_ERROR);
            return false;
        }
        $api->setAPIResource('virtual_machines/' . $this->virtual_machine_id . '/backups/' . $this->id);
        $result = $api->sendDELETERequest();
        $this->deleted = ($result['info']['http_code'] == 200);
        return $this->deleted;
    }

}

class CustomTemplate {

    public $template_id;
    public $label;
    public $description;
    public $vmid;
    public $backup_id;
    public $user_id;
    public $reseller;

    public function __construct($template_id = null) {
        $this->template_id = $template_id;
        $this->label = $label;
        $this->description = $description;
        $this->vmid = $vmid;
    }

    public function create($backup_id) {

        if ($this->vmid < 1) {
            throw new Exception("To call CustomTemplate::create() you must set it's id first");
        }
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->vmid . '/backups/' . $backup_id . '/convert');
        $res = $api->sendPOSTRequest();
        if ($res['info']['http_code'] === 200) {
            return $res['response'];
        }
        return false;
    }

}
/**
 * Upgrade Schedule class
 *
 * Allows management of Scheduled Upgrades
 */
class UpgradeSchedule {

    public $id = 0;
    public $label = '';
    public $extra_slices = 0;
    public $temporary = false;
    public $run_at;
    public $days;

    public function __construct($label, $extra_slices, $run_at, $days = 0) {
        $this->temporary = ($days > 0);
        $this->label = $label;
        $this->extra_slices = $extra_slices;
        $this->run_at = date_format('c', $run_at);
        if ($days > 0) {
            $this->days = $days;
        }
    }

}

/**
 * Virtual Machines class
 *
 * Allows management of Virtual Machines
 */
class VirtualMachine {

    public $label = '';
    public $hostname = '';
    public $domain_name = '';
    public $slices_count = 0;
    public $slices_required = 0;
    public $backups_enabled = 0;
    public $rsync_backups_enabled = 0;
    public $r1_soft_backups_enabled = 0;
    public $system_template_id = 0;
    public $cloud_id = 0;
    public $id;
    public $consumer_id = 0;
    public $created_at = null; // "2009-05-12T09:33:05-04:00"
    public $updated_at = null; // "2009-05-12T09:33:05-04:00"
    public $password = '';
    public $backups = array(); // call fullyLoad() to retrieve this
    public $licence = array();
    public $upgrade_schedules = array(); // call fullyLoad() to retrieve this
    public $deleted = false;
	public $auto_scaling = 0;

    public function __construct($label = '', $hostname = '', $slices_required = '', $backups_enabled = '', $rsync_backups_enabled='', $r1_soft_backups_enabled='', $cloud_id = '', $system_template_id = '', $consumer_id = 0, $licence=NULL, $storage_nodes_required = null, $ram_nodes_required = null,$auto_scaling = 0) {
        $this->label = $label;
        $this->hostname = $hostname;
        $this->slices_required = $slices_required;
        $this->storage_nodes_required = $storage_nodes_required;
        $this->ram_nodes_required = $ram_nodes_required;

        $this->backups_enabled = $backups_enabled;
        $this->rsync_backups_enabled = $rsync_backups_enabled;
        $this->r1_soft_backups_enabled = $r1_soft_backups_enabled;

        $this->cloud_id = $cloud_id;
        $this->system_template_id = $system_template_id;
        $this->consumer_id = $consumer_id;
        $this->licence = $licence;
		$this->auto_scaling = $auto_scaling;
    }

    private function _doAction($action) {
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/' . $action);

        $result = $api->sendPOSTRequest();
        if ($result['info']['http_code'] != 200) {
            throw new Exception("Error performing action. ".$result['errors']->errors);
        } else if (isset($result['response'])) {
            foreach ($result['response']->virtual_machine as $key => $value) {
                $this->$key = $value;
            }
        }
        return $this;
    }

    /**
     * Powers on a virtual machine
     * @return VirtualMachine Virtual Machine instance
     */
    public function powerOn() {
        try{
            $res = $this->_doAction('power_on');
        }catch(Exception $e){
            return $e->getMessage();
        }
        return $res;
    }

    /**
     * Powers off a virtual machine
     * @return VirtualMachine Virtual Machine instance
     */
    public function powerOff() {
        try{
            $res = $this->_doAction('power_off');
        }catch(Exception $e){
            return $e->getMessage();
        }
        return $res;
    }

    /**
     * Gracefully shuts down a virtual machine
     * @return VirtualMachine Virtual Machine instance
     */
    public function shutdown() {
        try{
            $res = $this->_doAction('shutdown');
        }catch(Exception $e){
            return $e->getMessage();
        }
        return $res;
    }

    /**
     * Reboots a virtual machine
     * @return VirtualMachine Virtual Machine instance
     */
    public function reboot() {
        try{
            $res = $this->_doAction('reboot');
        }catch(Exception $e){
            return $e->getMessage();
        }
        return $res;
    }

    /**
     * Creates a backup
     * @param string $label Name of backup
     * @return Backup Backup instance
     */
    public function createBackup($label) {
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/backups');
        $result = $api->sendPOSTRequest($json_request);
        if ($result['info']['http_code'] == 422) {
            $error = (array) $result['errors']->errors;
            $error = array_values($error);
            throw new Exception($error[0]);
        } else {
            $this->backups[] = $api->_castObjectToClass('Backup', $result['response']);
        }
        return $result['response'];
    }

    /**
     * Creates a temporary upgrade schedule
     * @param string $label Name of upgrade schedule
     * @param int $extra_slices Number of new nodes
     * @param date $run_at Date to run upgrade schedule
     * @param int $days Number of days to run upgrade schedule for
     * @return UpgradeSchedule instance
     */
    public function createTemporaryUpgradeSchedule($label, $extra_slices, $run_at, $days) {
        $bInputErrors = false;
        if (!is_string($label) || strlen($label) < 0) {
            trigger_error("To call VirtualMachine::createTemporaryUpgradeSchedule() you must specify a label", E_USER_ERROR);
            $bInputErrors = true;
        }
        if (!is_int($extra_slices)) {
            trigger_error("To call VirtualMachine::createTemporaryUpgradeSchedule() you must specify extra_slices as a number", E_USER_ERROR);
            $bInputErrors = true;
        }
        if (!is_int($days) || $days < 1) {
            trigger_error("To call VirtualMachine::createTemporaryUpgradeSchedule() you must specify days as a number greater than 0", E_USER_ERROR);
            $bInputErrors = true;
        }
        if ($bInputErrors)
            return false;

        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/backups');
        $json_request['backup']->label = $label;
        $result = $api->sendPOSTRequest($json_request);
        $return = null;
        if ($result['info']['http_code'] == 422) {
            // Do some error handling
        } else {
            $this->backups[] = $api->_castObjectToClass('Backup', $result['response']);
        }
        return $result['response'];
    }

    public function getNetworkUtilisation() {

        if (isset($this->NetworkUtilisation))
            return $this->NetworkUtilisation;

        // switch($period){
        //     case 'hourly' : 
        //         $from = time() - 60*60;
        //     case 'daily' : 
        //         $from =  mktime (0,0,0);
        //     case 'monthly' : 
        //         $from =  mktime (0,0,0, date("n"), 1);
        // }

        //getting monthly
        $from =  mktime (0,0,0, date("n")-1);
        $to = time();

        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/network_graph',true,'from='.$from.'&to='.$to);
        $result = $api->sendGETRequest();
        $response_body = $result['response'];
        $this->NetworkUtilisation = $response_body;
        return $response_body;
    }

    public function getCPUUsage() {

        if (isset($this->CPUUsage))
            return $this->CPUUsage;

        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/cpu_usage');
        $result = $api->sendGETRequest();
        $response_body = $result['response'];


        $this->CPUUsage = $response_body;


        return $response_body;
//         /virtual_machines/xx/network_utilisation.api10json
    }

    public function composeCPUUsageGraph($period='hourly') {

        $cu = $this->getCPUUsage();

        $name = "CPU Usage";

        $iName = $this->id . '-' . $period . '_cpu_usage.png';

        $DataSet = new pData;


//            foreach ($nu as $data) {
        $start = count($cu) - 48;

        if ($start < 0)
            $start = 0;

        for ($i = $start; $i < count($cu); $i++) {

            $data = $cu[$i];
            //print_r($i);

            $dsc = date('H:i', strtotime($data->created_at));

            if (($dsc == '00:00') || ($i == $start))
                $dsc = date('D H:i', strtotime($data->created_at));


            $DataSet->AddPoint((($data->cpu_time / $data->elapsed_time) * 10), "Serie1", $dsc);
        }


        $DataSet->AddAllSeries();
        $DataSet->SetAbsciseLabelSerie();
        $DataSet->SetYAxisUnit(' %');
        $DataSet->SetSerieName(formatBytes($received * 1024) . " received", "Serie1");
        // Initialise the graph
        $Test = new pChart(650, 280);

        $Test->setFontProperties(dirname(__FILE__) . "/assets/fonts/tahoma.ttf", 8);
        $Test->setGraphArea(35, 30, 640, 200);

        $Test->drawScale($DataSet->GetData(), $DataSet->GetDataDescription(), SCALE_NORMAL, 150, 150, 150, TRUE, 90, 0);
        $Test->drawGrid(4, TRUE, 230, 230, 230, 50); //reshotka
        // Draw the line graph
        $Test->drawLineGraph($DataSet->GetData(), $DataSet->GetDataDescription());
        $Test->drawPlotGraph($DataSet->GetData(), $DataSet->GetDataDescription(), 3, 2, 255, 255, 255);

        // Finish the graph

        $Test->setFontProperties(dirname(__FILE__) . "/assets/fonts/tahoma.ttf", 10);

        $Test->drawTitle(50, 22, $name, 50, 50, 50, 585);

        $Test->Render(dirname(__FILE__) . '/charts/' . $iName);
    }

    public function composeNetworkGraph($period='hourly') {

        $nu = $this->getNetworkUtilisation($period);

        $iName = $this->id . '-' . $period . '_usage.png';

        $DataSet = new pData;

        $received = 0;
        $sent = 0;

        $name = "";

        switch ($period) {

            case 'daily':

                $name = "Daily Usage";

                for ($i = date('G') - 23; $i <= date('G'); $i++) {

                    $start = mktime($i, 0, 0);

                    $end = mktime($i, 59, 59);

                    $dsc = date('G:i', $start);

                    $total_received = 0;
                    $total_sent = 0;

                    foreach ($nu as $data) {

                        $cur = strtotime($data->created_at);
                        //echo "$cur\n";
                        if (($cur >= $start) && ($cur <= $end)) {

                            $total_received = $total_received + $data->data_received;
                            $total_sent = $total_sent + $data->data_sent;

                            $received+=$data->data_received;
                            $sent+=$data->data_sent;
                        }
                    }

                    $DataSet->AddPoint((int) ($total_received / 1024), "Serie1", $dsc);
                    $DataSet->AddPoint((int) ($total_sent / 1024), "Serie2", $dsc);
                }
                break;

            case 'weekly':

                $name = "Weekly Usage";

                for ($i = -8; $i <= -1; $i++) {

                    $start = mktime(0, 0, 0, date('n'), date('d') + $i, date('Y'));

                    $end = mktime(23, 59, 59, date('n'), date('d') + $i, date('Y'));

                    $dsc = date('j D', $start);

                    $total_received = 0;
                    $total_sent = 0;

                    foreach ($nu as $data) {

                        $cur = strtotime($data->created_at);
                        //echo "$cur\n";
                        if (($cur >= $start) && ($cur <= $end)) {

                            $total_received = $total_received + $data->data_received;
                            $total_sent = $total_sent + $data->data_sent;

                            $received+=$data->data_received;
                            $sent+=$data->data_sent;
                        }
                    }

                    $DataSet->AddPoint((int) ($total_received / 1024), "Serie1", $dsc);
                    $DataSet->AddPoint((int) ($total_sent / 1024), "Serie2", $dsc);
                }

                break;    
            case 'monthly':

                $name = "Monthly Usage";

                for($i = date('j')-date('t')-1; $i < date('j'); $i++){

                    $start = mktime(0, 0, 0, date('n'), $i);

                    $end = mktime(0, 0, 0, date('n'), $i+1);

                    $dsc = date('j D', $start);

                    $total_received = 0;
                    $total_sent = 0;

                    foreach ($nu as $data) {
                        $cur = strtotime($data->created_at);

                        if (($cur >= $start) && ($cur <= $end)) {

                            $total_received += $data->data_received;
                            $total_sent += $data->data_sent;

                            $received+=$data->data_received;
                            $sent+=$data->data_sent;
                        }                        
                    }
                    $DataSet->AddPoint((int) ($total_received / 1024), "Serie1", $dsc);
                    $DataSet->AddPoint((int) ($total_sent / 1024), "Serie2", $dsc);
                }
                break;
        }

        $DataSet->AddAllSeries();
        $DataSet->SetAbsciseLabelSerie();

        $DataSet->SetYAxisUnit(' M');

        $DataSet->SetSerieName(formatBytes($received * 1024) . " received", "Serie1");
        $DataSet->SetSerieName(formatBytes($sent * 1024) . " sent", "Serie2");

        // Initialise the graph
        $Test = new pChart(650, 250);

        $Test->setFontProperties(dirname(__FILE__) . "/assets/fonts/tahoma.ttf", 8);
        $Test->setGraphArea(55, 30, 540, 200);

        $Test->drawScale($DataSet->GetData(), $DataSet->GetDataDescription(), SCALE_NORMAL, 150, 150, 150, TRUE, 90, 0);
        $Test->drawGrid(4, TRUE, 230, 230, 230, 50); //reshotka
        // Draw the line graph
        $Test->drawLineGraph($DataSet->GetData(), $DataSet->GetDataDescription());
        $Test->drawPlotGraph($DataSet->GetData(), $DataSet->GetDataDescription(), 3, 2, 255, 255, 255);


        // Finish the graph
        $Test->setFontProperties(dirname(__FILE__) . "/assets/fonts/tahoma.ttf", 8); // Description
        $Test->drawLegend(542, 30, $DataSet->GetDataDescription(), 255, 255, 255);

        $Test->setFontProperties(dirname(__FILE__) . "/assets/fonts/tahoma.ttf", 10);


        $Test->drawTitle(50, 22, $name, 50, 50, 50, 555);


        $Test->Render(dirname(__FILE__) . '/charts/' . $iName);
    }

    /**
     * Outputs a bandwidth usage graph to output stream
     * @param string $period Period of usage ('hourly', 'daily', 'weekly', 'monthly')
     */
    public function showNetworkGraph($period) {
        if (!in_array($period, array('hourly', 'daily', 'weekly', 'monthly', 'yearly'))) {
            trigger_error("To call VirtualMachine::getNetworkGraph() you must specify a period of hourly, daily, weekly or monthly", E_USER_ERROR);
            return false;
        }
        return $this->showGraph($period, 'network');
    }

    /**
     * Outputs a CPU usage graph to output stream
     * @param string $period Period of usage ('hourly', 'daily', 'weekly', 'monthly')
     */
    public function showCPUGraph($period) {
        if (!in_array($period, array('hourly', 'daily', 'weekly', 'monthly', 'yearly'))) {
            trigger_error("To call VirtualMachine::getCPUGraph() you must specify a period of hourly, daily, weekly or monthly", E_USER_ERROR);
            return false;
        }
        return $this->showGraph($period, 'cpu');
    }

    protected function showGraph($period, $type) {
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/' . $type . '_graph', false, 'period=' . $period);
        $result = $api->sendGETRequest();
        $response_body = $result['response_body'];
        header('Content-type: ' . $result['info']['content_type']);
        //header('Expires: ' . gmdate('D, d M Y H:i:s', time() + (2 * 3600)) . ' GMT');
        //header('Cache-Control: max-age=3600, public');
        echo ($result['response_body']);
    }

    public function getConsole() {

        $api = VPSNET::getInstance();

        $api->setAPIResource('virtual_machines/' . $this->id . '/console');
        $result = $api->sendGETRequest();

        if (is_object($result['response']))
            return $result['response']->session;

        else
            return false;
        //print_r($result);
        //return $result['response_body'];
    }

    /**
     * Outputs a Console to output stream
     */
    public function showConsole() {
        $api = VPSNET::getInstance();


//                print_r($result);
//                exit;
        $urlpath = substr($_SERVER['PATH_INFO'], 1);
//                $urlpath ='';
//                print_r($_SERVER);
//                echo 'virtual_machines/' . $this->id . '/console_proxy/' . $urlpath;
//		exit;

        $api->setAPIResource('virtual_machines/' . $this->id . '/console_proxy/' . $urlpath, false);


        //$response_body = $result['response_body'];
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $requestdata = 'k=' . urlencode($_POST['k']) . '&';
            $requestdata .= 'w=' . urlencode($_POST['w']) . '&';
            $requestdata .= 'c=' . urlencode($_POST['c']) . '&';
            $requestdata .= 'h=' . urlencode($_POST['h']) . '&';
            $requestdata .= 's=' . urlencode($_POST['s']) . '&';
            $result = $api->sendPOSTRequest($requestdata, false);
            header('Content-type: ' . $result['info']['content_type']);
            echo $result['response_body'];
        } else {
            $result = $api->sendGETRequest();


            if (strpos($urlpath, '.css')) {
                header('Content-type: text/css');
            } else {
                header('Content-type: ' . $result['info']['content_type']);
            }
            echo ($result['response_body']);
        }
        return $result;
    }

    /**
     * Retrieves a list of backups and adds it to backups property of current instance
     * @return array Array of Backups instances
     */
    public function loadBackups() {
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id . '/backups');
        $result = $api->sendGETRequest();
        if ($result['info']['http_code'] == 422) {
            // Do some error handling
        } else {
            $this->backups = array();
            $response = $result['response'];
            for ($x = 0; $x < count($response); $x++) {
                $this->backups[$x] = $api->_castObjectToClass('Backup', $response[$x]);
            }
        }
        return $this->backups;
    }

    public function loadCloud($id) {

        $api = VPSNET::getInstance();
        $api->setAPIResource('clouds/' . $id);
        $result = $api->sendGETRequest();

        return $result['response'];
        //print_r($result);
    }
	
	
	//load licenses
	public function loadLicenses()
	{
	    $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id.'/licenses');
        $result = $api->sendGETRequest();
        $this->backups =array();
        if ($result['info']['http_code'] == 422 || !isset($result['response'])) {
            // Do some error handling
        } else {
            foreach ($result['response']->backup_licenses as $key => $value) {
                $this->backups[$key] = $value;
            }
        }
        
        return $this->backups;
	}

    public function loadFully($full = true) {
        $api = VPSNET::getInstance();
        $api->setAPIResource('virtual_machines/' . $this->id);
        $result = $api->sendGETRequest();

        if ($result['info']['http_code'] == 422 || !isset($result['response'])) {
            // Do some error handling
        } else {
            foreach ($result['response']->virtual_machine as $key => $value) {
                $this->$key = $value;
            }
        }
        
        $vm->virtual_machine->id = $this->id;

        //var_dump($this);
//                print_r($this->LoadCloud($this->cloud_id));

        return $this;
    }

    public function getCpanel() {

        $cpanel = 'none';

//        print_r($this);

        if (is_array($this->virtual_machine_licenses))
            foreach ($this->virtual_machine_licenses as $linfo) {
                if ($linfo->virtual_machine_license->license_id == 1)
                    $cpanel = 'cpanel';
                if ($linfo->virtual_machine_license->license_id == 6)
                    $cpanel = 'isp manager';
            }

        return $cpanel;
    }
	
	public function getCpanelLicenceIds()
	{  
	   $licence_id = array();
	   if (is_array($this->virtual_machine_licenses))
	   {
            foreach ($this->virtual_machine_licenses as $linfo) 
			{
			   $licence_id[] = $linfo->virtual_machine_license->license_id;
			 }
	   }
	   return $licence_id;		
	}

    public function reinstall($system_template_id,$regular_nod=0,$ram_nod=0) {

        $api = VPSNET::getInstance();

        if ($this->id < 1) {
            throw new Exception("To call VirtualMachine::reinstall() you must set it's id first");
        }
        
        if ($this->running){
            try{
                $this->powerOff();
            }catch(Exception $e){
                return $e->getMessage();
            }
        }
            

        do {
            usleep(2000000);
            $this->loadFully();
        } while ($this->running);

        $api->setAPIResource('virtual_machines/' . $this->id . '/reinstall');
        $requestdata['system_template_id'] = $system_template_id;
	    $requestdata['slices_required'] = $regular_nod;
		$requestdata['daily_nodes_required'] = 0;
		$requestdata['ram_nodes_required'] = $ram_nod; 
        $json_request['virtual_machine'] = $requestdata;
		
        $result = $api->sendPUTRequest($json_request);
	    return ($result['info']['http_code'] == 200);
    }

    public function setConsumer($consumer_id = 0) {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            throw new Exception("To call VirtualMachine::update() you must set it's id first");
        }

        $this->consumer_id = $consumer_id;

        foreach ($api->getNodes(0, 'vps') as $node) {
            if ($node->virtual_machine_id == $this->id) {
                $node->update($consumer_id);
            }
        }

        foreach ($api->getNodes(0, 'storage') as $node) {
            if ($node->virtual_machine_id == $this->id) {
                $node->update($consumer_id);
            }
        }


        foreach ($api->getNodes(0, 'ram') as $node) {
            if ($node->virtual_machine_id == $this->id) {
                $node->update($consumer_id);
            }
        }

        return $this->update();
    }

    /**
     * Updates virtual machine
     * @return boolean True if update succeeded, false otherwise
     */
    public function update() {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            throw new Exception("To call VirtualMachine::update() you must set it's id first");
        }
        
        $api->setAPIResource('virtual_machines/' . $this->id);
        
        $requestdata['label'] = $this->label;
        $requestdata['slices_required'] = $this->slices_required ? $this->slices_required : $this->slices_count;
        $requestdata['consumer_id'] = $this->consumer_id;
        $requestdata['backups_enabled'] = (int) $this->backups_enabled;
        $requestdata['rsync_backups_enabled'] = (int) $this->rsync_backups_enabled;
        $requestdata['r1_soft_backups_enabled'] = (int) $this->r1_soft_backups_enabled;
        $requestdata['storage_nodes_required'] = !is_null($this->storage_nodes_required) ? $this->storage_nodes_required : $this->storage_nodes_count;
        $requestdata['ram_nodes_required'] = !is_null($this->ram_nodes_required) ?  $this->ram_nodes_required : $this->ram_nodes_count;
        $requestdata['auto_scaling'] = (int) $this->auto_scaling;
        if (is_array($this->licence))
            $requestdata['licenses'] = $this->licence;
        
        $json_request['virtual_machine'] = $requestdata;

        $result = $api->sendPUTRequest($json_request);
        
        if ($result['info']['http_code'] == 200) {
            return true;
        }
        if (isset($result['errors']) && isset($result['errors']->errors) && count($result['errors']->errors) > 0) {
            $errors = array();

            foreach ($result['errors']->errors as $error) {
                $errors[] = "{$error[0]}: {$error[1]}";
            }
            $errors = implode(", ", $errors);
            throw new Exception($errors);
        }
        throw new Exception("Unknown error");
    }

    /**
     * Removes a virtual machine
     * @return boolean true if virtual machine was removed, false otherwise
     */
    public function remove() {
        $api = VPSNET::getInstance();
        if ($this->id < 1) {
            trigger_error("To call VirtualMachine::remove() you must set its id", E_USER_ERROR);
            return false;
        }
        $api->setAPIResource('virtual_machines/' . $this->id);
        $result = $api->sendDELETERequest();



        //print_r($result);

        $this->deleted = ($result['info']['http_code'] == 200);
        return $this->deleted;
    }

    public function deleteIP($id) {
        $api = VPSNET::getInstance();

        $ips = $this->getIpAddresses();

        $ips = array_merge($ips['external'], $ips['internal']);

        foreach ($ips as $ip) {
            if ($id == $ip->id)
                $api->deleteIP($id);
//                print_r($ip);
        }
    }

    public function getIpAddresses() {
        $api = VPSNET::getInstance();

        $ips = $api->getIpAddresses();

        $return = array();

        $return['external'] = array();
        $return['internal'] = array();

        foreach ($ips as $ip) {

            if (
                    ($ip->virtual_machine_id == $this->id) ||
                    ($ip->description == 'for virtual machine #' . $this->id)
            )
                if ($ip->cloud_id)
                    $return['external'][] = $ip;
                else
                    $return['internal'][] = $ip;

//                print_r($ip);
        }

        return $return;
    }

}
